@@ -370,7 +370,7 @@ def name_addr_mappings_from_string(domain, addr_mappings, delegation_mapping, re
                     else:
                         for a_rrset in a_rrsets:
                             for a_rdata in a_rrset:
-                                mappings_from_file.append('%s=%s' % (lb2s(rdata.target.canonicalize().to_text()), IPAddr(a_rdata.address)))
+                                mappings_from_file.append('%s=[%s]' % (lb2s(rdata.target.canonicalize().to_text()), IPAddr(a_rdata.address)))
 
                 name_addr_mappings_from_string(domain, ','.join(mappings_from_file), delegation_mapping, require_name)
                 continue
