@@ -170,6 +170,8 @@ class DNSAuthGraph:
         self.next_nsec_id = 0
         self.next_rrset_id = 10
 
+        self._edge_keys = set()
+
     def _raphael_unit_mapping_expression(self, val, unit):
         #XXX doesn't work properly
         #if unit:
@@ -272,7 +274,7 @@ class DNSAuthGraph:
                     s += '.attr({%s})' % (','.join(attrs))
                 s += ';\n'
                 if node_id is not None and node_id in self.node_info:
-                    s += '\tthis.addNodeEvent(el, node_info[\'%s\']);\n' % node_id.replace('--', '\\-\\-')
+                    s += '\tthis.addNodeEvent(el, node_info[\'%s\']);\n' % node_id.replace('\\', '\\\\').replace('--', '\\-\\-')
 
         for i in range(node.childNodes.length):
             s += self._write_raphael_node(node.childNodes[i], node_id, transform)
@@ -661,10 +663,9 @@ class DNSAuthGraph:
         if not signed_node_zone.endswith(dnskey_node_zone):
             attrs['constraint'] = 'false'
 
-        try:
-            edge = self.G.get_edge(signed_node, dnskey_node, edge_key)
-        except KeyError:
-            self.G.add_edge(signed_node, dnskey_node, label=edge_label, key=edge_key, id=edge_id, color=line_color, style=line_style, dir='back', **attrs)
+        if (signed_node, dnskey_node, edge_key) not in self._edge_keys:
+            self._edge_keys.add((signed_node, dnskey_node, edge_key))
+            self.G.add_edge(signed_node, dnskey_node, label=edge_label, id=edge_id, color=line_color, style=line_style, dir='back', **attrs)
 
         consolidate_clients = name_obj.single_client()
         rrsig_serialized = rrsig_status.serialize(consolidate_clients=consolidate_clients, html_format=True)
@@ -889,16 +890,16 @@ class DNSAuthGraph:
 
         edge_id = 'dname-%s|%s|%s|%s' % (cname_node, dname_node, line_color.lstrip('#'), line_style)
         edge_key = '%s-%s' % (line_color, line_style)
-        try:
-            edge = self.G.get_edge(cname_node, dname_node, edge_key)
-        except KeyError:
+        if (cname_node, dname_node, edge_key) not in self._edge_keys:
+            self._edge_keys.add((cname_node, dname_node, edge_key))
+
             edge_label = ''
             if dname_status.errors:
                 edge_label = '<<TABLE BORDER="0"><TR><TD><IMG SCALE="TRUE" SRC="%s"/></TD></TR></TABLE>>' % ERROR_ICON
             elif dname_status.warnings:
                 edge_label = '<<TABLE BORDER="0"><TR><TD><IMG SCALE="TRUE" SRC="%s"/></TD></TR></TABLE>>' % WARNING_ICON
 
-            self.G.add_edge(cname_node, dname_node, label=edge_label, key=edge_key, id=edge_id, color=line_color, style=line_style, dir='back')
+            self.G.add_edge(cname_node, dname_node, label=edge_label, id=edge_id, color=line_color, style=line_style, dir='back')
             self.node_info[edge_id] = [dname_status.serialize(html_format=True)]
 
         if edge_id not in self.node_mapping:
@@ -1071,7 +1072,7 @@ class DNSAuthGraph:
                 rrsig_status = name_obj.rrsig_status[rrset_info][rrsig][dnskey]
                 self.add_rrsig(rrsig_status, name_obj, signer_obj, signed_node, port=port)
 
-    def graph_rrset_auth(self, name_obj, name, rdtype):
+    def graph_rrset_auth(self, name_obj, name, rdtype, trace=None):
         if (name, rdtype) not in self.processed_rrsets:
             self.processed_rrsets[(name, rdtype)] = []
 
@@ -1100,14 +1101,19 @@ class DNSAuthGraph:
         if name_obj.is_zone() and rdtype in (dns.rdatatype.DNSKEY, dns.rdatatype.DS):
             return []
 
+        # trace is used just for CNAME chains
+        if trace is None:
+            trace = [name]
+
         cname_nodes = []
-        # if this name is a alias, then graph the CNAME, unless this is a
-        # recursive analysis
+        # if this name is an alias, then graph its target, i.e., the canonical
+        # name, unless this is a recursive analysis.
         if name_obj.analysis_type != ANALYSIS_TYPE_RECURSIVE:
             if name in name_obj.cname_targets:
                 for target, cname_obj in name_obj.cname_targets[name].items():
                     if cname_obj is not None:
-                        cname_nodes.extend(self.graph_rrset_auth(cname_obj, target, rdtype))
+                        if target not in trace:
+                            cname_nodes.extend(self.graph_rrset_auth(cname_obj, target, rdtype, trace + [target]))
 
         query = name_obj.queries[(name, rdtype)]
         node_to_cname_mapping = set()
